const globalClearTimeout = clearTimeout;
const globalClearInterval = clearInterval;
const globalClearImmediate = clearImmediate;
const promisifySymbol = Symbol.for('nodejs.util.promisify.custom');
/**
 * The mock clock implementation
 */
export class Clock {
    #now = 1;
    #timers = new Map();
    get timers() {
        return this.#timers;
    }
    Date;
    #saved;
    /**
     * Schedule a function to be run when the clock has advanced `n` ms beyond
     * the current point.
     *
     * Only ms granularity.
     */
    setTimeout;
    /**
     * Schedule a function to be run when the clock advances each multiple
     * of `n` past the current point.
     *
     * If multiple steps are advanced at once, for example doing
     * `c.setInterval(fn, 1) ; c.advance(1000)`, then it will only call the
     * function once.  This allows you to simulate clock jitter.
     *
     * Only ms granularity.
     */
    setInterval;
    /**
     * Schedule a function to be run the next time the clock is advanced by
     * any ammount.
     */
    setImmediate;
    constructor() {
        this.Date = MockDate(this);
        const setTimeoutPromise = (n, value, options = {}) => this.setTimeoutPromise(n, value, options);
        this.setTimeout = Object.assign(this.#setTimeout, {
            __promisify__: setTimeoutPromise,
            [promisifySymbol]: setTimeoutPromise,
        });
        const setIntervalPromise = (n = 1, value, options = {}) => this.setIntervalPromise(n, value, options);
        this.setInterval = Object.assign(this.#setInterval, {
            __promisify__: setIntervalPromise,
            [promisifySymbol]: setIntervalPromise,
        });
        const setImmediatePromise = (value, options = {}) => this.setImmediatePromise(value, options);
        this.setImmediate = Object.assign(this.#setImmediate, {
            __promisify__: setImmediatePromise,
            [promisifySymbol]: setImmediatePromise,
        });
    }
    /**
     * Returns the current ms time on the clock.
     */
    now() {
        return Math.floor(this.#now);
    }
    /**
     * Set the clock to a specific time.  Will fire timers that you zoom past.
     */
    travel(to) {
        this.#now = to;
        this.advance(0);
    }
    /**
     * Advance the clock by `n` ms.  Use floats for smaller increments of time.
     */
    advance(n) {
        this.#now += n;
        for (const [w, t] of this.timers.entries()) {
            if (w <= this.#now) {
                this.timers.delete(w);
                t.forEach(({ _onTimeout: f }) => f && f());
            }
        }
    }
    /**
     * Advance the clock in steps, awaiting a Promise at each step, so that
     * actual asynchronous events can occur, as well as timers.
     */
    async flow(n, step = 5) {
        do {
            this.advance(step);
            await Promise.resolve();
        } while (0 <= (n -= step));
    }
    #setImmediate(f, ...a) {
        const fn = a.length ? () => f(...a) : f;
        return new Timer(this, this.now(), fn);
    }
    async setImmediatePromise(value, options = {}) {
        const { signal, reffed = true } = options;
        if (signal?.aborted) {
            /* c8 ignore start */
            throw signal.reason || new Error('The operation was aborted');
            /* c8 ignore stop */
        }
        let res;
        let rej;
        const p = new Promise((s, j) => {
            res = s;
            rej = j;
        });
        const timer = this.setImmediate(() => res(value));
        if (!reffed)
            timer.unref();
        if (!signal)
            return p;
        const cancel = () => {
            this.clearTimeout(timer);
            rej(
            /* c8 ignore start */
            signal.reason || new Error('The operation was aborted')
            /* c8 ignore stop */
            );
        };
        signal.addEventListener('abort', cancel, { once: true });
        return p.then(v => {
            signal.removeEventListener('abort', cancel);
            return v;
        });
    }
    #setTimeout(f, n = 1, ...a) {
        n = Math.max(n, 1);
        const fn = a.length ? () => f(...a) : f;
        const w = n + this.now();
        return new Timer(this, w, fn);
    }
    async setTimeoutPromise(n, value, options = {}) {
        const { signal, reffed = true } = options;
        if (signal?.aborted) {
            /* c8 ignore start */
            throw signal.reason || new Error('The operation was aborted');
            /* c8 ignore stop */
        }
        let res;
        let rej;
        const p = new Promise((s, j) => {
            res = s;
            rej = j;
        });
        const timer = this.setTimeout(() => res(value), n);
        if (!reffed)
            timer.unref();
        if (!signal) {
            return p;
        }
        const cancel = () => {
            this.clearTimeout(timer);
            /* c8 ignore start */
            rej(signal.reason || new Error('The operation was aborted'));
            /* c8 ignore stop */
        };
        signal.addEventListener('abort', cancel, { once: true });
        return p.then(v => {
            signal.removeEventListener('abort', cancel);
            return v;
        });
    }
    /**
     * Clear a timeout created by the clock.
     */
    clearTimeout(t) {
        if (t) {
            if (typeof t.clear === 'function')
                t.clear();
            else
                globalClearTimeout(t);
        }
    }
    /**
     * Clear an interval created by the clock. (alias for
     * {@link Clock#clearTimeout})
     */
    clearInterval(t) {
        if (t) {
            if (typeof t.clear === 'function')
                t.clear();
            else
                globalClearInterval(t);
        }
    }
    /**
     * Clear an immediate timer created by the clock. (alias for
     * {@link Clock#clearTimeout})
     */
    clearImmediate(t) {
        if (t) {
            if (typeof t.clear === 'function')
                t.clear();
            else
                globalClearImmediate(t);
        }
    }
    #setInterval(f, n = 1, ...a) {
        n = Math.max(n, 1);
        const fn = a.length ? () => f(...a) : f;
        const t = this.setTimeout(() => {
            while (t.w <= this.#now) {
                t.w += n;
            }
            const timers = this.timers.get(t.w) || [];
            timers.push(t);
            this.timers.set(t.w, timers);
            fn();
        }, n);
        return t;
    }
    async *setIntervalPromise(n = 1, value, options = {}) {
        // the signal will just cause the timeout promise to reject,
        // so there's really not much to do here.
        while (true) {
            yield await this.setTimeoutPromise(n, value, options);
        }
    }
    /**
     * Mock of `process.hrtime()`, returning `[seconds, nanoseconds]` on the
     * clock.
     */
    hrtime(s = [0, 0]) {
        const t = s[0] * 1e3 + s[1] / 1e6;
        const r = this.#now - t;
        const sec = Math.floor(r / 1e3);
        const usec = Math.floor(((r / 1e3) % 1) * 1e9);
        return [sec, usec];
    }
    /**
     * Mock of `process.hrtime.bigint()`, returning BigInt representation of
     * current nanosecond time.
     */
    hrtimeBigint() {
        return BigInt(this.#now * 1e6);
    }
    /**
     * Mocks all the things in the global space.
     *
     * Returns exit function, for ease of doing `t.teardown(c.enter())`.
     */
    enter() {
        if (this.#saved) {
            /* c8 ignore start */
            return () => { };
            /* c8 ignore stop */
        }
        /* istanbul ignore next - backwards comp affordance */
        const { performance: { now } = require('perf_hooks').performance, Date, setTimeout, setInterval, setImmediate, clearTimeout, clearInterval, clearImmediate, process: { hrtime }, } = global;
        const saved = (this.#saved = {
            performance: { now },
            Date,
            setTimeout,
            setInterval,
            setImmediate,
            clearTimeout,
            clearInterval,
            clearImmediate,
            process: { hrtime },
        });
        const { performance: globalPerformance } = global;
        Object.defineProperty(globalPerformance, 'now', {
            value: () => this.#saved === saved ? this.#now : saved.performance.now,
            enumerable: true,
            configurable: true,
        });
        global.process.hrtime = Object.assign((a) => this.#saved === saved
            ? this.hrtime(a)
            : saved.process.hrtime(a), {
            bigint: () => this.#saved === saved
                ? this.hrtimeBigint()
                : saved.process.hrtime.bigint(),
        });
        const self = this;
        const { now: _now, ...dateProps } = global.Date;
        global.Date = Object.assign(function Date(...args) {
            const D = self.#saved === saved ? self.Date : saved.Date;
            return !new.target
                ? String(new D())
                : Reflect.construct(D, args);
        }, {
            ...dateProps,
            now: () => self.#saved === saved ? self.now() : saved.Date.now(),
        });
        const proxyPromisify = (fn, method) => {
            const who = () => (this.#saved === saved ? this : saved);
            const m = (p) => who()[method][p];
            const prop = (p) => ({
                get: () => m(p),
                configurable: true,
                enumerable: true,
            });
            return Object.defineProperties(fn, {
                __promisify__: prop('__promisify__'),
                [promisifySymbol]: prop(promisifySymbol),
            });
        };
        function setTimeoutProxy(fn, n, ...a) {
            return self.#saved === saved
                ? self.setTimeout(fn, n, ...a)
                : saved.setTimeout(fn, n, ...a);
        }
        global.setTimeout = proxyPromisify(setTimeoutProxy, 'setTimeout');
        function setIntervalProxy(f, n = 1, ...a) {
            return self.#saved === saved
                ? self.setInterval(f, n, ...a)
                : saved.setInterval(f, n, ...a);
        }
        global.setInterval = proxyPromisify(setIntervalProxy, 'setInterval');
        function setImmediateProxy(fn, ...a) {
            return self.#saved === saved
                ? self.setImmediate(fn, ...a)
                : saved.setImmediate(fn, ...a);
        }
        global.setImmediate = proxyPromisify(setImmediateProxy, 'setImmediate');
        global.clearTimeout = (t) => this.#saved === saved
            ? this.clearTimeout(t)
            : saved.clearTimeout(t);
        global.clearInterval = (t) => this.#saved === saved
            ? this.clearInterval(t)
            : saved.clearInterval(t);
        global.clearImmediate = (t) => this.#saved === saved
            ? this.clearImmediate(t)
            : saved.clearImmediate(t);
        return () => this.exit();
    }
    /**
     * If entered, exit the clock, restoring the global state
     */
    exit() {
        if (!this.#saved) {
            return false;
        }
        const { performance: { now }, Date, setTimeout, setInterval, clearTimeout, clearInterval, clearImmediate, process: { hrtime }, } = this.#saved;
        this.#saved = undefined;
        const { performance: globalPerformance } = global;
        Object.defineProperty(globalPerformance, 'now', {
            value: now,
            enumerable: true,
            configurable: true,
        });
        Object.assign(global.process, { hrtime });
        Object.assign(global, {
            Date,
            setTimeout,
            setInterval,
            clearTimeout,
            clearInterval,
            clearImmediate,
        });
        return true;
    }
}
/**
 * Create a class like `Date`, but with its `now` value tied
 * to the value of a {@link Clock} instance.
 */
export const MockDate = (clock) => {
    return class MockDate extends Date {
        constructor(...args) {
            if (!args.length)
                args.push(clock.now());
            super(...args);
        }
        static now() {
            return clock.now();
        }
    };
};
/**
 * A Timer that lives in the mock clock
 */
export class Timer {
    /**
     * {@link Clock} governing this timer
     */
    clock;
    /**
     * When this timer should fire
     */
    w;
    /**
     * use the property name `_onTimeout` so that the global `clearTimeout`
     * method in Node will make it no-op if it gets passed there.
     */
    _onTimeout;
    /**
     * set by {@link Timer#unref}
     *
     * These timers don't keep the event loop open anyway, so this doesn't
     * do much, but it's useful when testing to ensure that a timeout
     * had .ref() or .unref() called on it.
     */
    reffed = true;
    constructor(clock, w, f) {
        this._onTimeout = f;
        this.w = Math.floor(w);
        this.clock = clock;
        const timers = clock.timers.get(w) || [];
        timers.push(this);
        clock.timers.set(w, timers);
    }
    /**
     * simulacrum of node's Timer.unref. Just sets the {@link Timer#ref} field.
     */
    unref() {
        this.reffed = false;
    }
    /**
     * simulacrum of node's Timer.ref. Just sets the {@link Timer#ref} field.
     */
    ref() {
        this.reffed = true;
    }
    /**
     * Remove this timer from the clock's queue.
     */
    clear() {
        this._onTimeout = null;
        const list = this.clock.timers.get(this.w);
        if (list && list.length) {
            this.clock.timers.set(this.w, list.filter(t => t !== this));
        }
    }
}
//# sourceMappingURL=index.js.map